# Copyright (c) 2000-2002 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""
unittest for the visitors.diadefs module
"""

__revision__ = "$Id: unittest_inspector.py,v 1.2 2004/02/17 07:39:45 syt Exp $"

import unittest
import sys

from logilab.common import astng
from logilab.common.astng import ASTNGManager
from logilab.pyreverse.visitors import inspector

def astng_wrapper(func, modname):
    return func(modname)

project = ASTNGManager().project_from_files(['data'], astng_wrapper)
        
class LinkerTC(unittest.TestCase):
    def setUp(self):
        self.linker = inspector.Linker(project)
        self.linker.visit(project)

    def test_class_ancestors(self):
        klass = project.get_module('data.clientmodule_test').locals['Specialization']
        self.assert_(hasattr(klass, 'baseobjects'))
        self.assertEqual(len(klass.baseobjects), 1)
        self.assert_(isinstance(klass.baseobjects[0], astng.Class))
        self.assertEqual(klass.baseobjects[0].name, "Ancestor")
        
    def test_class_implements(self):
        klass = project.get_module('data.clientmodule_test').locals['Ancestor']
        self.assert_(hasattr(klass, 'implements'))
        self.assertEqual(len(klass.implements), 1)
        self.assert_(isinstance(klass.implements[0], astng.Class))
        self.assertEqual(klass.implements[0].name, "Interface")
        klass = project.get_module('data.clientmodule_test').locals['Specialization']
        self.assert_(hasattr(klass, 'implements'))
        self.assertEqual(len(klass.implements), 0)
        
    def test_locals_assignment_resolution(self):
        klass = project.get_module('data.clientmodule_test').locals['Specialization']
        self.assert_(hasattr(klass, 'locals_type'))
        type_dict = klass.locals_type
        self.assertEqual(len(type_dict), 2)
        keys = type_dict.keys()
        keys.sort()
        self.assertEqual(keys, ['TYPE', 'top'])
        self.assertEqual(type_dict['TYPE'], 'final class')
        self.assertEqual(type_dict['top'], 'class')
        
    def test_instance_attrs_resolution(self):
        klass = project.get_module('data.clientmodule_test').locals['Specialization']
        self.assert_(hasattr(klass, 'instance_attrs_type'))
        type_dict = klass.instance_attrs_type
        self.assertEqual(len(type_dict), 2)
        keys = type_dict.keys()
        keys.sort()
        self.assertEqual(keys, ['_id', 'relation'])
        self.assert_(isinstance(type_dict['_id'], astng.Function))
        self.assertEqual(type_dict['_id'].name, '__init__')
        self.assert_(isinstance(type_dict['relation'], astng.Class), type_dict['relation'])
        self.assertEqual(type_dict['relation'].name, 'DoNothing')
        
        
        
def suite():
    """return the unitest suite"""
    loader = unittest.TestLoader()
    module = sys.modules[__name__]
    if __name__ == '__main__' and len(sys.argv) > 1:
        return loader.loadTestsFromNames(sys.argv[1:], module)
    return loader.loadTestsFromModule(module)

def Run(runner=None):
    """run tests"""
    testsuite = suite()
    if runner is None:
        runner = unittest.TextTestRunner()
        # uncomment next line to write tests results in a file
        #runner.__init__(open('tests.log','w+'))    
    return runner.run(testsuite)
        
if __name__ == '__main__':
    Run()
